package com.iconplus.smartproc.controller;

import com.iconplus.smartproc.helper.model.EmptyResponse;
import com.iconplus.smartproc.model.request.RksDaftarRequest;
import com.iconplus.smartproc.model.request.RksDaftarIsiRequest;
import com.iconplus.smartproc.model.response.*;
import com.iconplus.smartproc.service.rksdaftarisi.*;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.web.bind.annotation.*;

@CrossOrigin(origins = "${fe.server}", allowCredentials = "true")
@RestController
@RequestMapping("/api/rks/daftar")
public class RksDaftarIsiController {

    private final GetListRksDaftarIsiService getListRksDaftarIsiService;
    private final PostCreateRksDaftarIsiService postCreateRksDaftarIsiService;
    private final GetListRksIsiService getListRksIsiService;
    private final DeleteRksDaftarIsiService deleteRksDaftarIsiService;
    private final PutUpdateRksDaftarIsiService putUpdateRksDaftarIsiService;
    private final GetListRksParentService getListRksParentService;
    private final PostCreateRksIsiService postCreateRksIsiService;
    private final DeleteRksIsiService deleteRksIsiService;
    private final PutEditRksIsiService putEditRksIsiService;
    public RksDaftarIsiController(GetListRksDaftarIsiService getListRksDaftarIsiService,
                                  PostCreateRksDaftarIsiService postCreateRksDaftarIsiService,
                                  GetListRksIsiService getListRksIsiService,
                                  DeleteRksDaftarIsiService deleteRksDaftarIsiService,
                                  PutUpdateRksDaftarIsiService putUpdateRksDaftarIsiService,
                                  GetListRksParentService getListRksParentService,
                                  PostCreateRksIsiService postCreateRksIsiService,
                                  DeleteRksIsiService deleteRksIsiService,
                                  PutEditRksIsiService putEditRksIsiService) {
        this.getListRksDaftarIsiService = getListRksDaftarIsiService;
        this.postCreateRksDaftarIsiService = postCreateRksDaftarIsiService;
        this.getListRksIsiService = getListRksIsiService;
        this.deleteRksDaftarIsiService = deleteRksDaftarIsiService;
        this.putUpdateRksDaftarIsiService = putUpdateRksDaftarIsiService;
        this.getListRksParentService = getListRksParentService;
        this.postCreateRksIsiService = postCreateRksIsiService;
        this.deleteRksIsiService = deleteRksIsiService;
        this.putEditRksIsiService = putEditRksIsiService;
    }


    @GetMapping()
    public ListRksDaftarResponse getAllRksDaftarIsi(@RequestParam(name = "page", defaultValue = "1") Integer page,
                                                    @RequestParam(name = "size", defaultValue = "5") Integer size){

        Pageable pageable = PageRequest.of((page - 1), size);
        RksDaftarRequest rksDaftarRequest = RksDaftarRequest.builder()
                .pageable(pageable)
                .build();

        return getListRksDaftarIsiService.execute(rksDaftarRequest);

    }

    @PostMapping()
    public RksDaftarResponse createRksDaftarIsi(@RequestBody RksDaftarRequest rksDaftarRequest) {
        return postCreateRksDaftarIsiService.execute(rksDaftarRequest);
    }

    @PutMapping("/{id}")
    public RksDaftarResponse editRksDaftarIsi(@PathVariable(name = "id") Long id,
                                              @RequestBody RksDaftarRequest rksDaftarRequest) {
        rksDaftarRequest.setId(id);
        return putUpdateRksDaftarIsiService.execute(rksDaftarRequest);
    }

    @DeleteMapping("/{id}")
    public EmptyResponse deleteRksDaftarIsi(@PathVariable(name = "id") Long id) {
        return deleteRksDaftarIsiService.execute(RksDaftarRequest.builder()
                .id(id)
                .build());

    }

    @GetMapping("/{id}/isi")
    public ListRksDaftarIsiResponse getListRksIsi(@PathVariable(name = "id") Long id,
                                                  @RequestParam(name = "page", defaultValue = "1") Integer page,
                                                  @RequestParam(name = "size", defaultValue = "5") Integer size) {

        Pageable pageable = PageRequest.of((page - 1), size);
        RksDaftarIsiRequest rksDaftarIsiRequest = RksDaftarIsiRequest.builder()
                .rksDaftarId(id)
                .pageable(pageable)
                .build();
        return getListRksIsiService.execute(rksDaftarIsiRequest);
    }

    @GetMapping("/{id}/parent")
    public ListRksDaftarIsiResponse getParentRksIsi(@PathVariable(name = "id") Long id) {
        return getListRksParentService.execute(RksDaftarIsiRequest.builder()
                .rksDaftarId(id)
                .build());
    }

    @PostMapping("/{id}/isi")
    public RksDaftarIsiResponse createRksIsi(@PathVariable(name = "id") Long id,
                                             @RequestBody RksDaftarIsiRequest rksDaftarIsiRequest) {
        rksDaftarIsiRequest.setRksDaftarId(id);
        return postCreateRksIsiService.execute(rksDaftarIsiRequest);
    }

    @PutMapping("/isi/{id}")
    public RksDaftarIsiResponse editRksIsi(@PathVariable(name = "id") Long id,
                                           @RequestBody RksDaftarIsiRequest rksDaftarIsiRequest) {
        rksDaftarIsiRequest.setId(id);
        return putEditRksIsiService.execute(rksDaftarIsiRequest);
    }

    @DeleteMapping("/isi/{id}")
    public EmptyResponse deleteRksIsi(@PathVariable(name = "id") Long id) {
        return deleteRksIsiService.execute(RksDaftarIsiRequest.builder()
                .id(id)
                .build());
    }

}
