package com.iconplus.smartproc.service.rkstemplate;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.model.Pagination;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.RksTemplateVariabel;
import com.iconplus.smartproc.model.request.RksTemplateVariabelRequest;
import com.iconplus.smartproc.model.response.ListRksTemplateVariabelResponse;
import com.iconplus.smartproc.model.response.RksTemplateVariabelResponse;
import com.iconplus.smartproc.repository.RksTemplateVariabelRepository;
import com.iconplus.smartproc.util.Constants;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class GetListRksTemplateVariabelService implements BaseService<RksTemplateVariabelRequest, ListRksTemplateVariabelResponse> {

    private final RksTemplateVariabelRepository rksTemplateVariabelRepository;
    public GetListRksTemplateVariabelService(RksTemplateVariabelRepository rksTemplateVariabelRepository) {
        this.rksTemplateVariabelRepository = rksTemplateVariabelRepository;
    }

    @Override
    public ListRksTemplateVariabelResponse execute(RksTemplateVariabelRequest input) {

        List<RksTemplateVariabelResponse> rksTemplateVariabelResponseList = new ArrayList<>();
        var rksTemplateVariabelPage = rksTemplateVariabelRepository.findByRksTemplateIdAndIsDeleteFalse(input.getRksTemplateId(), input.getPageable());
        for (RksTemplateVariabel rksTemplateVariabel : rksTemplateVariabelPage) {
            RksTemplateVariabelResponse rksTemplateVariabelResponse = RksTemplateVariabelResponse.builder()
                    .id(rksTemplateVariabel.getId())
                    .rksTemplateId(rksTemplateVariabel.getRksTemplateId())
                    .variableName(rksTemplateVariabel.getVariableName())
                    .isian(rksTemplateVariabel.getIsian())
                    .keterangan(rksTemplateVariabel.getKeterangan())
                    .contohIsian(rksTemplateVariabel.getContohIsian())
                    .digunakanDi(rksTemplateVariabel.getDigunakanDi())
                    .build();

            rksTemplateVariabelResponseList.add(rksTemplateVariabelResponse);
        }


        return ListRksTemplateVariabelResponse.builder()
                .data(rksTemplateVariabelResponseList)
                .pagination(Pagination.builder()
                        .pageSize(input.getPageable().getPageSize())
                        .currentPage(input.getPageable().getPageNumber())
                        .totalPages(rksTemplateVariabelPage.getTotalPages())
                        .totalRecords(rksTemplateVariabelPage.getTotalElements())
                        .isFirstPage(rksTemplateVariabelPage.isFirst())
                        .isLastPage(rksTemplateVariabelPage.isLast())
                        .build())
                .build();
    }
}
