package com.iconplus.smartproc.service.rkstemplate;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.RksDaftarIsiRequest;
import com.iconplus.smartproc.model.request.RksTemplateRequest;
import com.iconplus.smartproc.model.response.ListRksDaftarIsiResponse;
import com.iconplus.smartproc.repository.RksDaftarIsiRepository;
import com.iconplus.smartproc.repository.RksTemplateRepository;
import com.iconplus.smartproc.service.rksdaftarisi.GetListRksIsiService;
import com.iconplus.smartproc.util.Constants;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

@Service
public class GetTemplateListRksDaftarIsiService implements BaseService<RksTemplateRequest, ListRksDaftarIsiResponse> {

    private final GetListRksIsiService getListRksIsiService;
    private final RksTemplateRepository rksTemplateRepository;
    public GetTemplateListRksDaftarIsiService(GetListRksIsiService getListRksIsiService,
                                              RksTemplateRepository rksTemplateRepository) {
        this.getListRksIsiService = getListRksIsiService;
        this.rksTemplateRepository = rksTemplateRepository;
    }

    @Override
    public ListRksDaftarIsiResponse execute(RksTemplateRequest input) {

        var rksTemplate = rksTemplateRepository.findByIdAndIsDeleteFalse(input.getId())
                .orElseThrow(() -> new BusinessException(
                        Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "RKS Template", input.getId())
                ));

        return getListRksIsiService.execute(RksDaftarIsiRequest.builder()
                .rksDaftarId(rksTemplate.getRksDaftarId())
                .pageable(input.getPageable())
                .build());
    }
}
