package com.iconplus.smartproc.service.rkstemplate;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.RksTemplateVariabel;
import com.iconplus.smartproc.model.request.RksTemplateVariabelRequest;
import com.iconplus.smartproc.model.response.RksTemplateVariabelResponse;
import com.iconplus.smartproc.repository.RksTemplateVariabelRepository;
import com.iconplus.smartproc.util.Constants;
import org.springframework.stereotype.Service;

@Service
public class PostCreateRksTemplateVariabelService implements BaseService<RksTemplateVariabelRequest, RksTemplateVariabelResponse> {

    private final RksTemplateVariabelRepository rksTemplateVariabelRepository;
    public PostCreateRksTemplateVariabelService(RksTemplateVariabelRepository rksTemplateVariabelRepository) {
        this.rksTemplateVariabelRepository = rksTemplateVariabelRepository;
    }

    @Override
    public RksTemplateVariabelResponse execute(RksTemplateVariabelRequest input) {

        var rksTemplateVariabel= rksTemplateVariabelRepository.findByVariableNameAndIsDeleteFalse(input.getVariableName());
        if (rksTemplateVariabel.isPresent()) {
            throw new BusinessException(Constants.ERR_CODE_10002,
                    Constants.ERR_TTL_10002,
                    String.format(Constants.ERR_MSG_10002, "RKS Template Variabel", input.getVariableName()));
        }

        RksTemplateVariabel rksTemplateVariabelEntity = RksTemplateVariabel.builder()
                .rksTemplateId(input.getRksTemplateId())
                .variableName(input.getVariableName())
                .keterangan(input.getKeterangan())
                .contohIsian(input.getContohIsian())
                .digunakanDi(input.getDigunakanDi())
                .isian(input.getIsian())
                .isActive(input.getIsActive())
                .isDelete(false)
                .build();

        var result = rksTemplateVariabelRepository.save(rksTemplateVariabelEntity);

        return RksTemplateVariabelResponse.builder()
                .id(result.getId())
                .build();
    }
}
